##================  BITRASH (rev 20190829)  ================##
##================     by Astro The Fox     ================##
##================       (source code)      ================##

##======== imports

## importing the gui wraper
from appJar import gui
import os
import sys
import string
import random
from inc import presetmk
from inc import presetld
from inc import engine

##======== globals

## global program settings
paramCount = 17 ## count of the preset parameters, needed to check if the preset is good
blep = "BITRASH : ERROR" ## header of error messages
fontsize = 7 ## main font size to use
fontfam = "Arial" ## main font family

## session variables

counterPasses = 0 ## counter of times "corrupt" button has been pressed in a row, needed for autonumbering

## get current working path into a var

pathWorking = False
if getattr(sys, 'frozen', False):
        pathWorking = sys._MEIPASS
elif __file__:
        pathWorking = os.path.dirname(os.path.realpath(__file__))

## throw error if no good working directory

if (not pathWorking):
        raise Exception("bad working path!")

## resource file locations

res = "res" ## name of the resource folder
imgLogo = os.path.join(pathWorking, res, "logo.gif")
imgAbout = os.path.join(pathWorking, res, "abt.gif")
imgIcon = os.path.join(pathWorking, res, "favicon.ico")
initPreset = os.path.join(pathWorking, res, "init.btrsh")

## globally accessible path variables, used by file selection fields, provided to corruption engine as args
pathOpenFile = ""
pathSaveFile = ""
pathSaveName = ""

##======== descriptLists

dlistByteplace = ["BYTEPLACE", "BYTEPLACE replaces each Nth byte with theuser-input value", 2, "N", "VAL"]
dlistSwapper = ["SWAPPER", "SWAPPER swaps two adjacent bytes places at every N bytes", 1, "N"]
dlistRchunk = ["RAND CHUNK", "RAND CHUNK fills last X% of CYCLE bytes with random values", 2, "CYCLE", "X"]
dlistAdder = ["ADDER", "ADDER adds INCR to each Nth byte", 2, "N", "INCR"]
dlistShifter = ["BIT SHIFTER", "BIT SHIFTER shifts each Nth byte to DIR by 1 bit (0=L, 1=R)", 2, "N", "DIR"]
dlistRandplace = ["RANDPLACE", "RANDPLACE replaces every Nth byte with a random value", 1, "N"]

namesList = ["byteplace", "swapper", "rchunk", "adder", "randplace" , "shifter"]
dlistlist = [dlistByteplace, dlistSwapper, dlistRchunk, dlistAdder, dlistRandplace, dlistShifter]

## dictionary of local entries+checkboxes and a separate one for chboxes
## (oh god am i doing this its gonna be SO bulky)
entries = []
checkboxes = []

##================              ================##
##================ FUNCTION DEF ================##
##================              ================##

##======== name fun

def recombinePath(nameSource):
	point = nameSource.find(".", (len(nameSource)-5), len(nameSource))
	path = nameSource[0:point]
	ext = nameSource[point+1:]
	nameDest = path + "-BITRASHED." + ext
	return nameDest

def recombineName(nameSource):
	point = nameSource.find(".", (len(nameSource)-5), len(nameSource))
	slash = nameSource.rfind("/")
	name = nameSource[slash+1:point]
	ext = nameSource[point+1:]
	mark = "-BITRASHED"
	nameDest = name + mark + "." + ext
	return nameDest

def numberName(nameSource):
	point = nameSource.find(".", (len(nameSource)-5), len(nameSource))
	name = nameSource[0:point]
	ext = nameSource[point+1:]
	
	global counterPasses
	if (counterPasses == 1):
		nameDest = name + "--" + str(counterPasses) + "." + ext
	else:
		nameDest = name[:-len(str(counterPasses-1))] + str(counterPasses) + "." + ext
	return nameDest

def getExtention(nameSource):
	point = nameSource.find(".", (len(nameSource)-5), len(nameSource))
	path = nameSource[0:point]
	ext = nameSource[point+1:]
	return ext

def getPath(nameSource):
	slash = nameSource.rfind("/")
	path = nameSource[0:slash]
	return path

##======== PRESET management

## pushes a value to target device
def pushEntryValue(target, value):
	temp = ""
	chbx = True ## checkbox flag
	if (value == -1): ## if val is 'preset true' then True
		temp = True
	elif (value == -2): ## same with false
		temp = False
	else:
		temp = str(value) ## if it is not -1 or -2 it means it is an entry value, not checkbox
		chbx = False ## set checkbox to false

	if (chbx == True): ## if it is a checkbox - set it by name
		app.setCheckBox(target, temp)
	else: ## else set entry by name
		app.setEntry(target, temp)

## pulls a value from target device
def pullEntryValue(target):
	temp = ""
	try:
		temp = app.getEntry(target) ## first we try to read an entry by name
	except:
		temp = app.getCheckBox(target) ## if we fail to do so, read checkbox


	## exceptions that allow float or negative:
	exc = []
	## if we didn't get BOOL (checkbox) OR an exception field
	if ((not type(temp) == type(True)) and (not target in exc)):
		#and if we either didn't get a number or got a negative number
		if ((temp.isdigit() and (float(temp) < 0)) or (not temp.isdigit())):
			## return BAD marker
			return "BAD"
                
	## else process the value to distinguish entry/checkbox type values
	if (temp == True): ## if got true, convert to -1 'preset true'
		return -1
	elif (temp == False): ## if got false convert to -2 'preset false'
		return -2
	else: ## else we got a string, so we int() it
		return int(app.getEntry(target))

## puts a list of parameters to GUI fields
def listToGUI(parameters):
	for i in range(len(entries)):
		pushEntryValue(entries[i], parameters[i])

## drags parameters from GUI, returns a list of them
def GUIToList():
	params = []
	for en in entries:
		params.append(pullEntryValue(en))
	return params

## load preset routine
def loadPreset(file):
	params = presetld.loadPreset(paramCount, file)
	listToGUI(params)

## save preset routine
def savePreset(file):
	params = GUIToList()
	presetmk.makePreset(params, file)

## verify preset routine
def verifyPreset():
	isGood = True
	list = GUIToList()
	for item in list:
		if (item == "BAD"):
			app.warningBox(blep, "Some of the parameters are not set correctly. All fields only allow positive numeric values.")
			isGood = False
	return isGood

## return a list of random parameters
def genRandomList():
        params = []
        hell = random.randrange(16, 512)
        params.append(200+random.randrange(1000))       ## preserve head
        params.append(random.randint(-2, -1))           ## byteplace on
        params.append(32+hell*random.randrange(160))    ## byteplace freq
        params.append(random.randint(0, 255))           ## byteplace val
        params.append(random.randint(-2, -1))           ## swapper on
        params.append(32+hell*random.randrange(100))    ## swapper freq
        params.append(random.randint(-2, -1))           ## rand chunk on
        params.append(32+hell*random.randrange(480))    ## rand chunk cycle
        params.append(random.randint(1,15))             ## rand chunk percent
        params.append(random.randint(-2, -1))           ## adder on
        params.append(32+hell*random.randrange(100))    ## adder freq
        params.append(random.randint(0, 128))           ## adder val
        params.append(random.randint(-2, -1))           ## randplace on
        params.append(32+hell*random.randrange(200))    ## randplace freq
        params.append(random.randint(-2, -1))           ## shifter on
        params.append(32+hell*random.randrange(130))    ## shifter freq
        params.append(random.randint(0, 1))             ## shifter dir
        return params

##======== updater of all global path variables with momentary on screen values

def updateLines(arg):
	global pathOpenFile
	global pathSaveFile
	global pathSaveName
	pof = app.getEntry("entrySource")
	psf = app.getEntry("entryDestDir")
	psn = app.getEntry("New file name: ")
	pathOpenFile = pof
	pathSaveFile = psf
	pathSaveName = psn
	return

#======== corrupt engine call function

def corrupt(originalPath, dirToSave, nameToSave):
	## here we check if the provided data exists at all...
	if ((originalPath == "") or (dirToSave == "") or (nameToSave == "")):
		app.warningBox(blep, "You didn't provide some file data, please do so and try again!")
		return -1
	if (not os.path.isfile(originalPath)):
		app.warningBox(blep, "Source file not found! Maybe it was removed or deleted. Choose a file and try again!")
		return -2
	
	newPath = os.path.join(dirToSave, nameToSave)
	engine.mainProcess(originalPath, newPath, GUIToList())
	return 0

##======== GUI binders

def openp(name):
	pat = (app.openBox(title="CHOOSE DIRECTORY...", dirName=pathWorking, asFile=False, fileTypes=[('BITRASH presets', '*.btrsh')]))
	if (not pat):
		return -1
	pathLoadPreset = pat
	loadPreset(pat)
	return 0

def initp(name):
	pat = initPreset
	pathLoadPreset = pat
	loadPreset(pat)
	return

def randp(name):
	preset = genRandomList()
	listToGUI(preset)
	return

def savep(name):
	if (not verifyPreset()):
		return
	pat = app.directoryBox(title="CHOOSE DIRECTORY...", dirName=pathWorking)
	if (not pat):
		return -1
	file = app.stringBox("INPUT NEW NAME...", "name for a new file:", parent=None)
	if (not file):
		return -1
	fp = pat + "/" + file + ".btrsh"
	savePreset(fp)
	return 0

def openf(line):
        ## connect global vars from way up in the code
	global pathOpenFile
	global pathSaveFile
	global pathSaveName

	## put stuff from GUI into vars
	cont = app.getEntry(line)
	path = getPath(cont)
	name = recombineName(cont)
	pathSaveFile = path
	pathSaveName = name
	pushEntryValue("New file name: ",name)
	pushEntryValue("entryDestDir",path)
	pathOpenFile = cont
	
	## drop the autonumbering counter on new file load
	global counterPasses
	counterPasses = 0
	
	return

def makef(name):
	global pathOpenFile
	global pathSaveFile
	global pathSaveName

	if (not verifyPreset()): ## if the preset is entered incorrectly then return away
		return 1

	global counterPasses
	if (app.getCheckBox("enableNumbering") == True): ## if autonumbering is enabled - increase the counter and number the file
                counterPasses += 1
                pathSaveName = numberName(pathSaveName)

	corrupt(pathOpenFile, pathSaveFile, pathSaveName)

##======== GUI button -> function binder function

def actButtsHandler(name):
	if (name == "SAVE PRESET"):
		savep(name)
	elif (name == "LOAD PRESET"):
		openp(name)
	elif (name == "DEFAULTS"):
		initp(name)
	elif (name == "C  O  R  R  U  P  T"):
		makef(name)
	elif (name == "ABOUT"):
		showAbout()
	elif (name == "DONE"):
		hideAbout()
	elif (name == "BACK"):
		hideMore()
	elif (name == "QUIT"):
                app.stop()
	elif (name == "RANDOMIZE"):
                randp(name)
	elif (name == "MORE"):
                showMore()
	return

##======== about subwindow show/hide

def showAbout():
        app.showSubWindow("ABOUT")
        app.showSubWindow("ABOUT")
        return

def hideAbout():
        app.hideSubWindow("ABOUT")
        return

##======== more opts subwindow show/hide

def showMore():
        app.showSubWindow("MORE")
        app.showSubWindow("MORE")
        return

def hideMore():
        app.hideSubWindow("MORE")

##======== GUI building functions

## echoes a bunch of appjar elements representing one trasher section
def echoTrasher(pos, name, dlist):
	retRows = 4
	app.setSticky("nesw")
	app.addHorizontalSeparator(pos, colour="#EEEEEE", colspan=4)
	app.setSticky("")
	app.addLabel(name+"Desc", dlist[1],pos+1, 0, colspan=4)
	app.setSticky("w")
	app.addNamedCheckBox("enabled", name+"Enable", pos+2, colspan=4)
	entries.append(name+"Enable")
	checkboxes.append(name+"Enable")
	opts = dlist[2]
	i = 0
	while (i < opts):
		app.setSticky("")
		app.addEntry(name+dlist[3+i], pos+3, 0 + 2 * i)
		app.setSticky("w")
		app.addLabel(name+dlist[3+i]+"Desc", dlist[3+i], pos+3, 1 + 2 * i)
		entries.append(name+dlist[3+i])
		i += 1
	return retRows

##================              ================##
##================ THE GUI PART ================##
##================              ================##

##======== gui setup

## initial setup
app = gui("BITRASH", showIcon=False)
app.winIcon = imgIcon
app.setIcon(app.winIcon)
app.showIcon = True
app.setResizable(canResize = False)
app.setBg("#202020")
app.setFg("#FFFFFF")
app.setFont(size=fontsize, family=fontfam)

curRow = 0

##======== the main window

## header
app.setSticky("")
app.addImage("picLogo", imgLogo, curRow, colspan=4)
curRow += 1

## files paths and directories
app.setSticky("nesw")
app.addHorizontalSeparator(curRow, colour="#EEEEEE", colspan=4)
app.addOpenEntry("entrySource", curRow, colspan=4)
app.setEntryChangeFunction("entrySource", openf)
app.addDirectoryEntry("entryDestDir",curRow+1, colspan=4)
app.setEntryChangeFunction("entryDestDir", updateLines)
app.addLabelEntry("New file name: ", curRow+2, colspan=4)
app.setEntryChangeFunction("New file name: ", updateLines)
curRow += 3

## preset load/save
app.setSticky("ne")
app.addButtons(["SAVE PRESET", "LOAD PRESET", "DEFAULTS", "RANDOMIZE", "MORE", "ABOUT", "QUIT"], actButtsHandler, curRow, colspan=4)
curRow += 1

## bit preserve section
app.setSticky("nesw")
app.addHorizontalSeparator(curRow, colour="#EEEEEE", colspan=4)
app.addLabelEntry("First bytes to preserve:", curRow+1)
curRow += 2
entries.append("First bytes to preserve:")

## trashers section
counter = 0
for item in dlistlist:
	curRow += echoTrasher(curRow, namesList[counter], dlistlist[counter])
	counter += 1

## corrupt buton
app.setSticky("nesw")
app.addHorizontalSeparator(curRow, colour="#EEEEEE", colspan=4)
app.setSticky("")
app.addButton("C  O  R  R  U  P  T", actButtsHandler, curRow+1, colspan=4)

## finalize by setting sizes abd colors of everything

for box in checkboxes:
        app.setCheckBoxFg(box, "#FF0000")

##======== the about subwindow

app.startSubWindow("ABOUT", modal=False)
app.setSticky("")
app.setBg("#202020")
app.setResizable(canResize=False)
app.addImage("picAbout", imgAbout, curRow, colspan=4)
app.setSticky("nesw")
app.addButton("DONE", actButtsHandler)
app.stopSubWindow()

##======== the more options subwindow

app.startSubWindow("MORE", modal=False)
app.setSticky("nesw")
app.setBg("#202020")
app.setResizable(canResize=False)
app.addNamedCheckBox("Enable auto-numbering // will auto-add a pass number at the end of the file.", "enableNumbering")
app.setCheckBoxFg("enableNumbering", "#FF0000")
app.addButton("BACK", actButtsHandler)
app.stopSubWindow()



##======== start

initp("i just had to put some arg here because of appjar ok")
app.go()
